<?php

namespace Snog\Flags;

use XF\AddOn\AbstractSetup;
use XF\AddOn\StepRunnerInstallTrait;
use XF\AddOn\StepRunnerUninstallTrait;
use XF\AddOn\StepRunnerUpgradeTrait;
use XF\Db\Schema\Alter;
use XF\Util\File;

class Setup extends AbstractSetup
{
	use StepRunnerInstallTrait;
	use StepRunnerUpgradeTrait;
	use StepRunnerUninstallTrait;

	public function checkRequirements(&$errors = [], &$warnings = [])
	{
		$versionId = $this->addOn->version_id;

		if($versionId && $versionId < '22')
		{
			$errors[] = 'Upgrades can only be done to version 1.2.2 or later';
		}

		if(!function_exists('bcadd'))
		{
			$errors[] = 'BC Math is either not installed on your server or PHP is not compiled with it enabled.';
		}

		if(!function_exists('gzopen'))
		{
			$errors[] = 'Gzip is either not installed on your server or is not configured for use with PHP.';
		}

		$datapath = \XF::config('externalDataPath');

		if (!is_writable($datapath))
		{
			$errors[] = 'The data folder is not able to be written to. Change the permissions to world writable (777)';
		}

		return $errors;
	}

	public function installStep1()
	{
		$this->schemaManager()->alterTable('xf_user', function(Alter $table)
		{
			$table->addColumn('snog_flag', 'VARCHAR', 3)->setDefault('');
		});
	}

	public function installStep2()
	{
		$this->schemaManager()->alterTable('xf_user_privacy', function(Alter $table)
		{
			$table->addColumn('snog_flag_view', 'enum')->values(['everyone', 'members', 'none'])->setDefault('none');
		});
	}

	public function installStep3()
	{
		$src = 'src/addons/Snog/Flags/defaultdata';
		$this->copyContents($src);
	}

	public function installStep4()
	{
		$tempDir = FILE::getTempDir();

		$tempPath = $tempDir . DIRECTORY_SEPARATOR . 'GeoLite2-Country.mmdb.gz';
		$unzippedFile = str_ireplace('.gz','', $tempPath);
		$finalPath = 'data://countryflags/GeoLite2-Country.mmdb';

		try
		{
			// Attempt to connect to MaxMind for the database
			$response = \XF::app()->http()->client()->get('http://geolite.maxmind.com/download/geoip/database/GeoLite2-Country.mmdb.gz');
			file_put_contents($tempPath, (string)$response->getBody());
		}
		catch(\GuzzleHttp\Exception\RequestException $e)
		{
			throw $this->exception($this->error(\XF::phrase('Error downloading and extracting GeoLite2 database.')));
		}

		$buffer = 4096;
		$zip = gzopen($tempPath, 'rb');

		if($zip !== false)
		{
			$fp = fopen($unzippedFile, 'wb');

			while(!gzeof($zip))
			{
				fwrite($fp, gzread($zip, $buffer));
			}

			fclose($fp);
			gzclose($zip);
			unlink($tempPath);

			File::copyFileToAbstractedPath($unzippedFile, $finalPath);
			unlink($unzippedFile);
		}else{
			throw $this->exception($this->error(\XF::phrase('Error downloading and extracting GeoLite2 database.')));
		}
	}

	public function upgrade2000370Step1()
	{
		$this->schemaManager()->alterTable('xf_user_privacy', function(Alter $table)
		{
			$table->changeColumn('snog_flag_view', 'enum')->values(['everyone', 'members', 'none'])->setDefault('none');
		});
	}

	public function uninstallStep1()
	{
		$this->schemaManager()->alterTable('xf_user', function(Alter $table)
		{
			$table->dropColumns(['snog_flag']);
		});
	}

	public function uninstallStep2()
	{
		$this->schemaManager()->alterTable('xf_user_privacy', function(Alter $table)
		{
			$table->dropColumns(['snog_flag_view']);
		});
	}

	public function copyContents($src, $sub = false)
	{
		if($sub) $basePath = str_ireplace('src/addons/Snog/Flags/defaultdata/', '', $src);
		$dir = opendir($src);

		while(false !== ( $file = readdir($dir)) )
		{
			if(($file != '.') && ($file != '..'))
			{
				if(is_dir($src . '/' . $file) )
				{
					$newSrc = $src . '/' . $file;
					$this->copyContents($newSrc, true);
				}else{
					$oldPath = $src . '/'. $file;

					if($sub)
					{
						$newFile = $basePath . '/' . $file;
					}else{
						$newFile = $file;
					}

					$newPath = sprintf('data://countryflags/%s', $newFile);
					File::copyFileToAbstractedPath($oldPath, $newPath);
				}
			}
		}

		closedir($dir);
	}
}